#include <DNSServer.h>
#include <esp_wifi.h>           
#include <AsyncTCP.h>           
#include <ESPAsyncWebServer.h>  
#include <WebSocketsServer.h>
#include <ArduinoJson.h>
#include <GyverFilters.h>

const size_t capacity = 16384;
DynamicJsonDocument doc(capacity);
WebSocketsServer webSocket = WebSocketsServer(11181);

const char *ssid = "LIGHTMETER";
const char *password = "4815162342";

#define MAX_CLIENTS 4   
#define WIFI_CHANNEL 6  

#define ANALOG_IN_PIN 34
#define LIMIT_MAX_ENV_ILLUMINANCE 80   
#define LIMIT_MIN_ILLUMINANCE 150      
#define LIMIT_MAX_ILLUMINANCE 950      
#define ENV_NUMBER_CALIBRATION_MEASURES 500
#define ENV_MEASURE_INTERVAL 1             
#define PRINT_INFO_INTERVAL 200            

long timer_2 = 0;
long timer_5 = 0;

int m = 0;
int x = 0;
int y = 0;

bool arr_filled = false;

int point_arr[320] = {};
int mid_point_arr[320] = {};
int point_arr_filt[320] = {};

byte x1_min = 0;
byte x1_max = 0;

float kp1 = 0;
float kp2 = 0;

long sum_E = 0;
int env_E = 0;  
long env_E_sum = 0;
int Emax = 0;
int Emin = 0;

int delta_y = 40;
int y_min = 0;

int Emax_sum = 0;
int Emin_sum = 0;
int sum_E_sum = 0;
int n_m = 0;

long timer_print = 0;
long start_time = 0;

GFilterRA average(0.5, 80);

const IPAddress localIP(192, 168, 8, 1);       
const IPAddress gatewayIP(192, 168, 8, 1);     
const IPAddress subnetMask(255, 255, 255, 0);  

const String localIPURL = "http://192.168.8.1"; 

const char index_html[] PROGMEM = R"=====(
  <!DOCTYPE html> <html>
    <head>
      <meta charset='UTF-8'>
      <title>LIGHTMETER</title>
      <style>
        body {background-color:#CCCCCC;}
        h1 {color:blue;}
      </style>
      <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body onload='init(); next=true;'>
      <h2>А хуле нам, безработным инженерам?...</h2>
      <h1>Хуйняметр!</h1>demid.net
      <table border=1 cellspacing=0 width=100%>
        <tr><td colspan=3><canvas id='c1' width='320' height='240'></canvas></td></tr>
      </table>
      <br>
      <table border=0 cellspacing=0 width=100%>
        <tr><td width='25%'><b>Условия</b></td><td colspan=2><span id='cond'></span></td></tr>
        <tr><td>&nbsp;</td></tr>
        <tr><td><b>Emax =</b></td><td><span id='Emax'></span></td></tr>
        <tr><td><b>Emin =</b></td><td><span id='Emin'></span></td></tr>
        <tr><td>&nbsp;</td></tr>
        <tr><td><b>kp1 =</b></td><td><span id='kp1'></span></td></tr>
        <tr><td><b>kp2 =</b></td><td><span id='kp2'></span></td></tr>
        <tr><td>&nbsp;</td></tr>
        <tr><td><b>Свет</b></td><td colspan=2><span id='result'></span></td></tr>
      </table>
      <script type='text/javascript'>
        var Socket;
        var x = 0;
        var ctx = c1.getContext('2d');
        ctx.globalCompositeOperation = 'lighter';
        ctx.shadowColor = 'black';
        ctx.strokeStyle = 'black';        
        function init() {
          Socket = new WebSocket('ws://' + window.location.hostname + ':11181/');
          Socket.onmessage = function(event){
            var res = JSON.parse(event.data);
            if(res.result==0) {result.innerHTML = 'ахуенный!'; result.style.color='green';}
            if(res.result==1) {result.innerHTML = 'сойдет'; result.style.color='yellow';}
            if(res.result==2) {result.innerHTML = 'че-то не фонтан'; result.style.color='orange';}
            if(res.result==3) {result.innerHTML = 'ваще говно...'; result.style.color='red';}
            if(res.cond==0) {cond.innerHTML = 'дохуя темно'; cond.style.color='red'; result.innerHTML = 'сделай посветлее!'; result.style.color='red';}
            if(res.cond==1) {cond.innerHTML = 'нормальные'; cond.style.color='green';}
            if(res.cond==2) {cond.innerHTML = 'дохуя светло'; cond.style.color='red'; result.innerHTML = 'сделай потемнее!'; result.style.color='red';}
            Emax.innerHTML =res.Emax;
            Emin.innerHTML =res.Emin;
            kp1.innerHTML =res.kp1/10 + ' %';
            kp2.innerHTML =res.kp2/10 + ' %';
            ctx.beginPath();
            ctx.clearRect(0,0,320,240);
            ctx.moveTo(5, 240-res.m[0]/17);
            for (var i = 1; i < 320; i++) {
              ctx.lineTo(5+i, 240-res.m[i]/17);
            }
            ctx.stroke();
          }
        }
      </script>
    </body>
  </html>
)=====";

DNSServer dnsServer;
AsyncWebServer server(80);

void setup() {  
  WiFi.mode(WIFI_AP);
  WiFi.softAPConfig(localIP, gatewayIP, subnetMask);
  WiFi.softAP(ssid, password, WIFI_CHANNEL, 0, MAX_CLIENTS);
  dnsServer.setTTL(300);   
  dnsServer.start(53, "*", localIP); 
  esp_wifi_stop();
  esp_wifi_deinit();
  wifi_init_config_t my_config = WIFI_INIT_CONFIG_DEFAULT();  
  my_config.ampdu_rx_enable = false;                          
  esp_wifi_init(&my_config); 
  esp_wifi_start();          
  delay(100);                
  server.on("/connecttest.txt", [](AsyncWebServerRequest *request) {
    request->redirect("http://logout.net");
  });  
  server.on("/wpad.dat", [](AsyncWebServerRequest *request) {
    request->send(404);
  });  

  server.on("/generate_204", [](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });  // android captive portal redirect
  server.on("/redirect", [](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });  //microsoft redirect
  server.on("/hotspot-detect.html", [](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });  //apple call home
  server.on("/canonical.html", [](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });  //firefox captive portal call home
  server.on("/success.txt", [](AsyncWebServerRequest *request) {
    request->send(200);
  });  //firefox captive portal call home
  server.on("/ncsi.txt", [](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });  //windows call home
  server.on("/favicon.ico", [](AsyncWebServerRequest *request) {
    request->send(404);
  });  //webpage icon
  server.on("/", HTTP_ANY, [](AsyncWebServerRequest *request) {
    AsyncWebServerResponse *response = request->beginResponse(200, "text/html", index_html);
    response->addHeader("Cache-Control", "public,max-age=31536000"); 
    request->send(response);
  });
  server.onNotFound([](AsyncWebServerRequest *request) {
    request->redirect(localIPURL);
  });

  server.begin();
  pinMode(ANALOG_IN_PIN, INPUT);
  webSocket.begin();
  env_E = 0;
}

void loop() {
  timer_2 = micros();
  if (x < 320) {
    arr_filled = false;
    point_arr[x] = analogRead(ANALOG_IN_PIN);
    point_arr_filt[x] = (int)average.filtered(point_arr[x]);
    if (x > 0) mid_point_arr[x] = (point_arr[x - 1] + point_arr[x]) / 2;
    x++;
  } else {
    arr_filled = true;
    x = 0;
    start_time = millis();
  }
  if (arr_filled == true) {
    sum_E = point_arr[0];
    x1_max = 0;
    x1_min = 0;
    for (int x1 = 1; x1 < 64; x1++) {
      sum_E += point_arr[x1] - env_E;
      if (point_arr[x1] > point_arr[x1_max])
        x1_max = x1;
      if (point_arr[x1] < point_arr[x1_min])
        x1_min = x1;
    }
    delta_y = 80 * ceil((point_arr[x1_max] - point_arr[x1_min]) / 80.0);
    y_min = point_arr[x1_max] - delta_y;
    if (y_min < 0)
      y_min = 0;
    Emax_sum += point_arr[x1_max];
    Emin_sum += point_arr[x1_min];
    sum_E_sum += sum_E;
    n_m++;
    if (millis() > (timer_print + PRINT_INFO_INTERVAL)) {
      timer_print = millis();
      Emax = Emax_sum / n_m;
      Emin = Emin_sum / n_m;
      kp1 = 100.0 * (Emax - Emin) / (Emax + Emin - 2 * env_E);
      if ((kp1 < 0) || (kp1 > 100))
        kp1 = 0.0;
      kp2 = 100.0 * (Emax - Emin) / (2 * (sum_E_sum / n_m) / 32);
      if (kp2 < 0.0)
        kp2 = 0.0;
      String mystr;
      if (kp1 < 5.0) {
        doc["result"] = 0;
      } else if ((kp1 >= 5.0) && (kp1 < 20.0)) {
        doc["result"] = 1;
      } else if ((kp1 >= 20.0) && (kp1 < 40.0)) {
        doc["result"] = 2;
      } else if (kp1 >= 40.0) {
        doc["result"] = 3;
      }

      if (Emax < 200)
        doc["cond"] = 0;
      else if (Emax > 3900)
        doc["cond"] = 2;
      else
        doc["cond"] = 1;

      Emax_sum = 0;
      Emin_sum = 0;
      sum_E_sum = 0;
      n_m = 0;
    }
    arr_filled = false;
    x = 0;

    doc["Emax"] = Emax;
    doc["Emin"] = Emin;
    doc["kp1"] = (int)(kp1 * 10);
    doc["kp2"] = (int)(kp2 * 10);
    for (int t = 0; t < 320; t++) {
      doc["m"][t] = point_arr[t];
    }
    String buf;
    serializeJson(doc, buf);
    webSocket.broadcastTXT(buf);
  }
  webSocket.loop();
  dnsServer.processNextRequest();  
}